function LayuiFileUpload(layui, viewer) {
    let $ = layui.$;
    let form = layui.form;
    let upload = layui.upload;
    if (!upload) {
        throw new Error('需要引入layui的upload模块');
    }
    let ajax = top.restAjax || top.ajax;
    let Viewer = viewer;
    let viewerObj = {};
    let style = `
    <style>
    /* 上传图片 */
    .form-upload-image { position: relative; display: inline-block; margin: 10px 10px 0 0; }
    .form-upload-image:last-child { margin-right: 0; }
    .form-upload-image img { height: 100px; }
    .form-upload-image .delete-btn { position: absolute; top: 0; right: 0; margin: 0; }
    .layui-form-item .upload-image-box { clear: both; width: 100%; padding: 0; display: block; }
    /* 上传附件 */
    .form-upload-file { }
    .form-upload-file .operation { text-align: center; }
    .form-upload-file .delete-btn { margin: 0; }
    /* 上传视频 */
    .form-upload-video { position: relative; display: inline-block; margin: 10px 10px 0 0; }
    .form-upload-video:last-child { margin-right: 0; }
    .form-upload-video .delete-btn { position: absolute; top: 0; right: 0; margin: 0; }
    /* 上传音频 */
    .form-upload-audio { position: relative; display: inline-block; margin: 10px 10px 0 0; }
    .form-upload-audio:last-child { margin-right: 0; }
    .form-upload-audio .delete-btn { position: absolute; top: 0; right: 0; margin: 0; }
    </style>
    `
    $(document.head).append(style);

    /**
     * 获取配置
     * @param opt
     * @return {{headers: *, fieldName: *, uploadBtnId: string, deleteBtnClass: string, fileBoxId: string, isApp: (*|boolean), id: string, isAppRelease: (*|boolean), maxCount: (number|*), isShow: (*|boolean)}}
     */
    function getOptData(opt) {
        let maxCount = !opt.maxCount || opt.maxCount < 0 ? 9 : opt.maxCount;
        let fieldName = opt.fieldName;
        let headers = opt.headers;
        headers = headers ? headers : {};
        let isApp = opt.isApp;
        let isAppRelease = opt.isAppRelease;
        let isShow = opt.isShow;

        let id = `#${fieldName}`;
        let fileBoxId = `${id}FileBox`;
        let uploadBtnId = `${id}UploadBtn`;
        let deleteBtnClass = `.delete-${fieldName}-btn`;
        return {
            maxCount: maxCount,
            fieldName: fieldName,
            headers: headers,
            isApp: isApp,
            isAppRelease: isAppRelease,
            isShow: isShow,
            id: id,
            fileBoxId: fileBoxId,
            uploadBtnId: uploadBtnId,
            deleteBtnClass: deleteBtnClass
        }
    }

    /**
     * 渲染
     * @param optData
     * @param fileArray
     * @param html
     */
    function render(optData, fileArray, html) {
        $(optData.fileBoxId).empty();
        $(optData.fileBoxId).append(html);

        if (fileArray.length < optData.maxCount) {
            $(optData.uploadBtnId).removeClass('layui-btn-disabled');
            $(optData.uploadBtnId).attr('disabled', false);
        } else {
            $(optData.uploadBtnId).addClass('layui-btn-disabled');
            $(optData.uploadBtnId).attr('disabled', true);
        }
    }

    /**
     * 初始化图片上传
     * @param opt
     */
    function initUploadImage(opt) {
        let optData = getOptData(opt);

        function init(callback) {
            let fileIds = $(optData.id).val();
            let fileArray = fileIds ? fileIds.split(',') : [];

            let html = '';
            for (let i = 0; i < fileArray.length; i++) {
                let fileId = fileArray[i];
                html += `
                    <div class="form-upload-image">
                        <img src="route/file/v2/download/true/${fileId}" align="加载失败">
                        <a class="layui-btn layui-btn-xs layui-btn-danger delete-btn delete-${optData.fieldName}-btn" href="javascript:void(0);" data-id="${fileId}" data-name="${optData.fieldName}">
                            <i class="fa fa-trash-o"></i>
                        </a>
                    </div>
                `;
            }
            render(optData, fileArray, html);
            callback ? callback() : '';
        }

        function addClick() {
            let layerLoadingIndex;
            let url = 'api/file/v2/upload-image';
            if (optData.isApp) {
                url = 'app/file/v2/upload-image'
            }
            if (optData.isAppRelease) {
                url = 'app/file/v2/upload-image-release'
            }
            upload.render({
                elem: optData.uploadBtnId,
                url: url,
                accept: 'images',
                acceptMime: 'image/*',
                field: 'image',
                exts: 'jpg|png|gif|bmp|jpeg',
                name: optData.fieldName,
                headers: optData.headers,
                before: function (obj) {
                    layerLoadingIndex = layer.msg('上传中...', {icon: 16, time: 0, shade: 0.3})
                },
                done: function (res, index, upload) {
                    layer.close(layerLoadingIndex);
                    let name = this.name;
                    let dom = $(`#${name}`);
                    let files = dom.val();
                    files = files ? files : '';
                    if (files.length > 0) {
                        files += ',';
                    }
                    files += res.data.fileId;
                    dom.val(files);
                    init(function () {
                        if (Viewer) {
                            viewerObj[name].update();
                        }
                    });
                },
                error: function (index, upload) {
                    layer.close(layerLoadingIndex);
                    layer.msg('文件上传失败');
                },
                progress: function (n, elem, res, index) {
                }
            });
            $(document).on('click', optData.deleteBtnClass, function () {
                let name = this.dataset.name;
                let id = this.dataset.id;
                let dom = $(`#${name}`);
                let files = dom.val().replace(id, '');
                files = files.replace(/\,+/g, ',');
                if (files.charAt(0) === ',') {
                    files = files.substring(1);
                }
                if (files.charAt(files.length - 1) === ',') {
                    files = files.substring(0, files.length - 1);
                }
                dom.val(files);
                init(function () {
                    if (Viewer) {
                        viewerObj[name].update();
                    }
                });
            });
        }

        init(function () {
            if (Viewer) {
                viewerObj[optData.fieldName] = new Viewer(document.getElementById(optData.fieldName + 'FileBox'), {navbar: false});
            }
        });

        if (!optData.isShow) {
            addClick();
        } else {
            $(optData.uploadBtnId).hide();
        }
    }

    /**
     * 初始化图片上传
     * @param fieldName
     * @param maxCount
     */
    this.initUploadImage = function (fieldName, maxCount) {
        initUploadImage({
            fieldName: fieldName,
            maxCount: maxCount,
            isApp: false,
            isAppRelease: false,
            isShow: false
        });
    }

    /**
     * 初始化APP图片上传
     * @param fieldName
     * @param maxCount
     * @param headers
     */
    this.initAppUploadImage = function (fieldName, maxCount, headers) {
        initUploadImage({
            fieldName: fieldName,
            maxCount: maxCount,
            isApp: true,
            isAppRelease: false,
            isShow: false,
            headers: headers
        });
    }

    /**
     * 初始化APP放行图片上传
     * @param fieldName
     * @param maxCount
     * @param headers
     */
    this.initAppReleaseUploadImage = function (fieldName, maxCount, headers) {
        initUploadImage({
            fieldName: fieldName,
            maxCount: maxCount,
            isApp: false,
            isAppRelease: true,
            isShow: false,
            headers: headers
        });
    }

    /**
     * 显示上传图片
     * @param fieldName
     * @param maxCount
     */
    this.initShowUploadImage = function (fieldName, maxCount) {
        initUploadImage({
            fieldName: fieldName,
            maxCount: maxCount,
            isApp: false,
            isAppRelease: false,
            isShow: true
        });
    }

    /**
     * 初始化文件上传
     * @param opt
     */
    function initUploadFile(opt) {
        let optData = getOptData(opt);

        function init() {
            let files = $(optData.id).val();
            let fileArray = files ? files.split(',') : [];

            let html = '';
            if (fileArray.length > 0) {
                let trs = '';
                for (let i = 0; i < fileArray.length; i++) {
                    let file = fileArray[i];
                    let idNameArray = file.split(':');
                    let fileId = idNameArray[0];
                    let fileName = idNameArray[1];
                    trs += `
                        <tr>
                           <td><a href="route/file/v2/download/false/${fileId}" target="_blank">${fileName}</a></td>
                           <td class="operation">
                               <button type="button" class="layui-btn layui-btn-xs layui-btn-danger delete-btn delete-${optData.fieldName}-btn" data-id="${fileId}" data-name="${fileName}" data-field-name="${optData.fieldName}">删除</button>
                           </td>
                        </tr>
                    `
                }
                html += `
                    <div class="form-upload-file">
                       <table class="layui-table" lay-size="sm">
                           <colgroup>
                               <col>
                               <col width="60">
                           </colgroup>
                           <thead>
                           <tr>
                               <th>文件名</th>
                               <th class="operation">操作</th>
                           </tr>
                           </thead>
                           <tbody>
                           ${trs}
                           </tbody>
                       </table>
                    </div>
                `;
            }

            render(optData, fileArray, html);
        }

        function addClick() {
            let layerLoadingIndex;
            let url = 'api/file/v2/upload-file';
            if (optData.isApp) {
                url = 'app/file/v2/upload-file'
            }
            if (optData.isAppRelease) {
                url = 'app/file/v2/upload-file-release'
            }
            upload.render({
                elem: optData.uploadBtnId,
                url: url,
                accept: 'file',
                acceptMime: [
                    'application/pdf',
                    'application/msword',
                    'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
                    'application/vnd.ms-excel',
                    'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
                    'application/vnd.ms-powerpoint',
                    'application/vnd.openxmlformats-officedocument.presentationml.presentation',
                    'application/vnd.ms-works',
                    'text/plain',
                    'application/x-rar',
                    'application/x-zip-compressed'
                ].join(','),
                field: 'file',
                exts: 'pdf|doc|docx|xls|xlsx|ppt|pptx|wps|txt|rar|zip',
                name: optData.fieldName,
                headers: optData.headers,
                before: function (obj) {
                    layerLoadingIndex = layer.msg('上传中...', {icon: 16, time: 0, shade: 0.3})
                },
                done: function (res, index, upload) {
                    layer.close(layerLoadingIndex);
                    let name = this.name;
                    let dom = $(`#${name}`);
                    let files = dom.val();
                    if (files.length > 0) {
                        files += ',';
                    }
                    files += res.data.fileId + ':' + res.data.fileName.replace(/\,/g, '，');
                    dom.val(files);
                    init();
                },
                error: function (index, upload) {
                    layer.close(layerLoadingIndex);
                    layer.msg('文件上传失败');
                },
                progress: function (n, elem, res, index) {
                }
            });
            $(document).on('click', optData.deleteBtnClass, function () {
                let id = this.dataset.id;
                let name = this.dataset.name;
                let fieldName = this.dataset.fieldName;
                let dom = $(`#${fieldName}`).val();
                let files = dom.val().replace(id + ':' + name, '');
                files = files.replace(/\,+/g, ',');
                if (files.charAt(0) === ',') {
                    files = files.substring(1);
                }
                if (files.charAt(files.length - 1) === ',') {
                    files = files.substring(0, files.length - 1);
                }
                dom.val(files);
                init();
            });
        }

        init();
        if (!optData.isShow) {
            addClick();
        } else {
            $(optData.uploadBtnId).hide();
        }

    }

    /**
     * 初始化文件上传
     * @param fieldName
     * @param maxCount
     */
    this.initUploadFile = function (fieldName, maxCount) {
        initUploadFile({
            fieldName: fieldName,
            maxCount: maxCount,
            isApp: false,
            isAppRelease: false,
            isShow: false
        })
    }

    /**
     * 初始化APP文件上传
     * @param fieldName
     * @param maxCount
     * @param headers
     */
    this.initAppUploadFile = function (fieldName, maxCount, headers) {
        initUploadFile({
            fieldName: fieldName,
            maxCount: maxCount,
            isApp: true,
            isAppRelease: false,
            isShow: false,
            headers: headers
        });
    }

    /**
     * 初始化APP放行文件上传
     * @param fieldName
     * @param maxCount
     * @param headers
     */
    this.initAppReleaseUploadFile = function (fieldName, maxCount, headers) {
        initUploadFile({
            fieldName: fieldName,
            maxCount: maxCount,
            isApp: false,
            isAppRelease: true,
            isShow: false,
            headers: headers
        });
    }

    /**
     * 显示上传文件
     * @param fieldName
     * @param maxCount
     */
    this.initShowUploadFile = function (fieldName, maxCount) {
        initUploadFile({
            fieldName: fieldName,
            maxCount: maxCount,
            isApp: false,
            isAppRelease: false,
            isShow: true
        });
    }

    /**
     * 初始化视频上传
     * @param opt
     */
    function initUploadVideo(opt) {
        let optData = getOptData(opt);

        function init() {
            let files = $(id).val();
            let fileArray = files ? files.split(',') : [];
            let html = '';
            for (let i = 0; i < fileArray.length; i++) {
                let file = fileArray[i];
                let idNameArray = file.split(':');
                let fileId = idNameArray[0];
                let fileName = idNameArray[1];
                html += `
                    <div class="form-upload-video">
                       <video width="320" height="240" controls>
                           <source src="route/file/v2/download/true/${fileId}" type="video/mp4">
                           您的浏览器不支持 video 标签
                       </video>
                       <a class="layui-btn layui-btn-xs layui-btn-danger delete-btn delete-${optData.fieldName}-btn" href="javascript:void(0);" data-id="${fileId}" data-name="${fileName}" data-field-name="${optData.fieldName}">
                           <i class="fa fa-trash-o"></i>
                       </a>
                    </div>
                `;
            }

            render(optData, fileArray, html)
        }

        function addClick() {
            let layerLoadingIndex;
            let url = 'api/file/v2/upload-video';
            if (optData.isApp) {
                url = 'app/file/v2/upload-video'
            }
            if (optData.isAppRelease) {
                url = 'app/file/v2/upload-video-release'
            }
            upload.render({
                elem: optData.uploadBtnId,
                url: url,
                accept: 'video',
                acceptMime: 'video/mp4',
                exts: 'mp4',
                field: 'video',
                name: optData.fieldName,
                headers: optData.headers,
                before: function (obj) {
                    layerLoadingIndex = layer.msg('上传中...', {icon: 16, time: 0, shade: 0.3})
                },
                done: function (res, index, upload) {
                    layer.close(layerLoadingIndex);
                    let name = this.name;
                    let dom = $(`#${name}`);
                    let files = dom.val();
                    if (files.length > 0) {
                        files += ',';
                    }
                    files += res.data.fileId + ':' + res.data.fileName.replace(/\,/g, '，');
                    dom.val(files);
                    init();
                },
                error: function (index, upload) {
                    layer.close(layerLoadingIndex);
                    layer.msg('文件上传失败');
                },
                progress: function (n, elem, res, index) {
                }
            });
            $(document).on('click', optData.deleteBtnClass, function () {
                let name = this.dataset.name;
                let id = this.dataset.id;
                let fieldName = this.dataset.fieldName;
                let dom = $(`#${fieldName}`);
                let files = dom.val().replace(id + ':' + name, '');
                files = files.replace(/\,+/g, ',');
                if (files.charAt(0) === ',') {
                    files = files.substring(1);
                }
                if (files.charAt(files.length - 1) === ',') {
                    files = files.substring(0, files.length - 1);
                }
                dom.val(files);
                init();
            });
        }

        init();

        if (!optData.isShow) {
            addClick();
        } else {
            $(optData.uploadBtnId).hide();
        }
    }

    /**
     * 初始化视频上传
     * @param fieldName
     * @param maxCount
     */
    this.initUploadVideo = function (fieldName, maxCount) {
        initUploadVideo({
            fieldName: fieldName,
            maxCount: maxCount,
            isApp: false,
            isAppRelease: false,
            isShow: false
        })
    }

    /**
     * 初始化APP视频上传
     * @param fieldName
     * @param maxCount
     * @param headers
     */
    this.initAppUploadVideo = function (fieldName, maxCount, headers) {
        initUploadVideo({
            fieldName: fieldName,
            maxCount: maxCount,
            isApp: true,
            isAppRelease: false,
            isShow: false,
            headers: headers
        });
    }

    /**
     * 初始化APP放行视频上传
     * @param fieldName
     * @param maxCount
     * @param headers
     */
    this.initAppReleaseUploadVideo = function (fieldName, maxCount, headers) {
        initUploadVideo({
            fieldName: fieldName,
            maxCount: maxCount,
            isApp: false,
            isAppRelease: true,
            isShow: false,
            headers: headers
        });
    }

    /**
     * 显示上传视频
     * @param fieldName
     * @param maxCount
     */
    this.initShowUploadVideo = function (fieldName, maxCount) {
        initUploadVideo({
            fieldName: fieldName,
            maxCount: maxCount,
            isApp: false,
            isAppRelease: false,
            isShow: true
        });
    }

    function initUploadAudio(opt) {
        let optData = getOptData(opt);

        function init() {
            let files = $(id).val();
            let fileArray = files ? files.split(',') : [];
            let html = '';
            for (let i = 0; i < fileArray.length; i++) {
                let file = fileArray[i];
                let idNameArray = file.split(':');
                let fileId = idNameArray[0];
                let fileName = idNameArray[1];
                html += `
                    <div class="form-upload-video">
                       <audio src="route/file/v2/download/true/${fileId}" controls>
                           您的浏览器不支持 audio 标签。
                       </audio>
                       <a class="layui-btn layui-btn-xs layui-btn-danger delete-btn delete-${optData.fieldName}-btn" href="javascript:void(0);" data-id="${fileId}" data-name="${fileName}" data-field-name="${optData.fieldName}">
                           <i class="fa fa-trash-o"></i>
                       </a>
                    </div>
                `;
            }

            render(optData, fileArray, html);
        }

        function addClick() {
            let layerLoadingIndex;
            let url = 'api/file/v2/upload-audio';
            if (optData.isApp) {
                url = 'app/file/v2/upload-audio'
            }
            if (optData.isAppRelease) {
                url = 'app/file/v2/upload-audio-release'
            }
            upload.render({
                elem: optData.uploadBtnId,
                url: url,
                accept: 'audio',
                acceptMime: [
                    'audio/wav',
                    'audio/mp3'
                ],
                exts: 'wav|mp3',
                field: 'audio',
                name: optData.fieldName,
                headers: optData.headers,
                before: function (obj) {
                    layerLoadingIndex = layer.msg('上传中...', {icon: 16, time: 0, shade: 0.3})
                },
                done: function (res, index, upload) {
                    layer.close(layerLoadingIndex);
                    let name = this.name;
                    let dom = $(`#${name}`);
                    let files = dom.val();
                    if (files.length > 0) {
                        files += ',';
                    }
                    files += res.data.fileId + ':' + res.data.fileName.replace(/\,/g, '，');
                    dom.val(files);
                    init();
                },
                error: function (index, upload) {
                    layer.close(layerLoadingIndex);
                    layer.msg('文件上传失败');
                },
                progress: function (n, elem, res, index) {
                }
            });
            $(document).on('click', optData.deleteBtnClass, function () {
                let name = this.dataset.name;
                let id = this.dataset.id;
                let fieldName = this.dataset.fieldName;
                let dom = $(`#${fieldName}`)
                let files = dom.val().replace(id + ':' + name, '');
                files = files.replace(/\,+/g, ',');
                if (files.charAt(0) === ',') {
                    files = files.substring(1);
                }
                if (files.charAt(files.length - 1) === ',') {
                    files = files.substring(0, files.length - 1);
                }
                dom.val(files);
                init();
            });
        }

        init();

        if (!optData.isShow) {
            addClick();
        } else {
            $(optData.uploadBtnId).hide();
        }
    }

    /**
     * 初始化音频上传
     * @param fieldName
     * @param maxCount
     */
    this.initUploadAudio = function (fieldName, maxCount) {
        initUploadAudio({
            fieldName: fieldName,
            maxCount: maxCount,
            isApp: false,
            isAppRelease: false,
            isShow: false
        })
    }

    /**
     * 初始化APP音频上传
     * @param fieldName
     * @param maxCount
     * @param headers
     */
    this.initAppUploadAudio = function (fieldName, maxCount, headers) {
        initUploadAudio({
            fieldName: fieldName,
            maxCount: maxCount,
            isApp: true,
            isAppRelease: false,
            isShow: false,
            headers: headers
        });
    }

    /**
     * 初始化APP放行音频上传
     * @param fieldName
     * @param maxCount
     * @param headers
     */
    this.initAppReleaseUploadAudio = function (fieldName, maxCount, headers) {
        initUploadAudio({
            fieldName: fieldName,
            maxCount: maxCount,
            isApp: false,
            isAppRelease: true,
            isShow: false,
            headers: headers
        });
    }

    /**
     * 显示上传音频
     * @param fieldName
     * @param maxCount
     */
    this.initShowUploadAudio = function (fieldName, maxCount) {
        initUploadAudio({
            fieldName: fieldName,
            maxCount: maxCount,
            isApp: false,
            isAppRelease: false,
            isShow: true
        });
    }

    /**
     * 清空上传字段
     * @param formData
     */
    this.clearUploadField = function (formData) {
        let fileInputs = $(formData.form).find('input[type="file"]');
        if (fileInputs.length === 0) {
            return;
        }
        $.each(fileInputs, function (index, item) {
            delete formData.field[item.name];
        });
    }

}