package ink.wgink.module.wechat.pay.pojo.v3.refund;

import ink.wgink.module.wechat.pay.enums.*;

import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName: Refunds
 * @Description: 退款
 * @Author: wanggeng
 * @Date: 2022/11/10 21:31
 * @Version: 1.0
 */
public class RefundResult {

    /**
     * 微信支付退款单号
     */
    private String refund_id;
    /**
     * 商户退款单号<br/>
     * 商户系统内部的退款单号，商户系统内部唯一，只能是数字、大小写字母_-|*@ ，同一退款单号多次请求只退一笔。
     */
    private String out_refund_no;
    /**
     * 微信支付交易订单号
     */
    private String transaction_id;
    /**
     * 商户订单号<br/>
     * 原支付交易对应的商户订单号
     */
    private String out_trade_no;
    /**
     * 退款渠道
     */
    private RefundChannelEnum channel;
    /**
     * 退款入账账户<br/>
     * 取当前退款单的退款入账方，有以下几种情况：<br/>
     * 1）退回银行卡：{银行名称}{卡类型}{卡尾号}<br/>
     * 2）退回支付用户零钱:支付用户零钱<br/>
     * 3）退还商户:商户基本账户商户结算银行账户<br/>
     * 4）退回支付用户零钱通:支付用户零钱通
     */
    private String user_received_account;
    /**
     * 退款成功时间<br/>
     * 退款成功时间，当退款状态为退款成功时有返回。
     */
    private String success_time;
    /**
     * 退款创建时间<br/>
     * 退款受理时间
     */
    private String create_time;
    /**
     * 退款状态<br/>
     * 退款到银行发现用户的卡作废或者冻结了，导致原路退款银行卡失败，可前往商户平台-交易中心，手动处理此笔退款。
     */
    private RefundStatusEnum status;
    /**
     * 资金账户<br/>
     * 退款所使用资金对应的资金账户类型
     */
    private RefundFundsAccountEnum funds_account;
    /**
     * 金额信息
     */
    private Amount amount;
    /**
     * 优惠退款信息
     */
    private PromotionDetail promotion_detail;

    public String getRefund_id() {
        return refund_id == null ? "" : refund_id.trim();
    }

    public void setRefund_id(String refund_id) {
        this.refund_id = refund_id;
    }

    public String getOut_refund_no() {
        return out_refund_no == null ? "" : out_refund_no.trim();
    }

    public void setOut_refund_no(String out_refund_no) {
        this.out_refund_no = out_refund_no;
    }

    public String getTransaction_id() {
        return transaction_id == null ? "" : transaction_id.trim();
    }

    public void setTransaction_id(String transaction_id) {
        this.transaction_id = transaction_id;
    }

    public String getOut_trade_no() {
        return out_trade_no == null ? "" : out_trade_no.trim();
    }

    public void setOut_trade_no(String out_trade_no) {
        this.out_trade_no = out_trade_no;
    }

    public RefundChannelEnum getChannel() {
        return channel;
    }

    public void setChannel(RefundChannelEnum channel) {
        this.channel = channel;
    }

    public String getUser_received_account() {
        return user_received_account == null ? "" : user_received_account.trim();
    }

    public void setUser_received_account(String user_received_account) {
        this.user_received_account = user_received_account;
    }

    public String getSuccess_time() {
        return success_time == null ? "" : success_time.trim();
    }

    public void setSuccess_time(String success_time) {
        this.success_time = success_time;
    }

    public String getCreate_time() {
        return create_time == null ? "" : create_time.trim();
    }

    public void setCreate_time(String create_time) {
        this.create_time = create_time;
    }

    public RefundStatusEnum getStatus() {
        return status;
    }

    public void setStatus(RefundStatusEnum status) {
        this.status = status;
    }

    public RefundFundsAccountEnum getFunds_account() {
        return funds_account;
    }

    public void setFunds_account(RefundFundsAccountEnum funds_account) {
        this.funds_account = funds_account;
    }

    public Amount getAmount() {
        return amount;
    }

    public void setAmount(Amount amount) {
        this.amount = amount;
    }

    public PromotionDetail getPromotion_detail() {
        return promotion_detail;
    }

    public void setPromotion_detail(PromotionDetail promotion_detail) {
        this.promotion_detail = promotion_detail;
    }

    /**
     * 金额信息
     */
    public static class Amount {
        /**
         * 订单金额<br/>
         * 订单总金额，单位为分
         */
        private Integer total;
        /**
         * 退款金额<br/>
         * 退款标价金额，单位为分，可以做部分退款
         */
        private Integer refund;
        /**
         * 退款出资账户及金额
         */
        private List<AmountFrom> from;
        /**
         * 用户支付金额<br/>
         * 现金支付金额，单位为分，只能为整数
         */
        private Integer payer_total;
        /**
         * 用户退款金额<br/>
         * 退款给用户的金额，不包含所有优惠券金额
         */
        private Integer payer_refund;
        /**
         * 应结退款金额<br/>
         * 去掉非充值代金券退款金额后的退款金额，单位为分，退款金额=申请退款金额-非充值代金券退款金额，退款金额<=申请退款金额
         */
        private Integer settlement_refund;
        /**
         * 应结订单金额<br/>
         * 应结订单金额=订单金额-免充值代金券金额，应结订单金额<=订单金额，单位为分
         */
        private Integer settlement_total;
        /**
         * 优惠退款金额<br/>
         * 优惠退款金额<=退款金额，退款金额-代金券或立减优惠退款金额为现金，说明详见代金券或立减优惠，单位为分
         */
        private Integer discount_refund;
        /**
         * 退款币种<br/>
         * 符合ISO 4217标准的三位字母代码，目前只支持人民币：CNY。
         */
        private String currency;
        /**
         * 手续费退款金额<br/>
         * 手续费退款金额，单位为分。
         */
        private Integer refund_fee;

        public Integer getTotal() {
            return total == null ? 0 : total;
        }

        public void setTotal(Integer total) {
            this.total = total;
        }

        public Integer getRefund() {
            return refund == null ? 0 : refund;
        }

        public void setRefund(Integer refund) {
            this.refund = refund;
        }

        public List<AmountFrom> getFrom() {
            return from == null ? new ArrayList() : from;
        }

        public void setFrom(List<AmountFrom> from) {
            this.from = from;
        }

        public Integer getPayer_total() {
            return payer_total == null ? 0 : payer_total;
        }

        public void setPayer_total(Integer payer_total) {
            this.payer_total = payer_total;
        }

        public Integer getPayer_refund() {
            return payer_refund == null ? 0 : payer_refund;
        }

        public void setPayer_refund(Integer payer_refund) {
            this.payer_refund = payer_refund;
        }

        public Integer getSettlement_refund() {
            return settlement_refund == null ? 0 : settlement_refund;
        }

        public void setSettlement_refund(Integer settlement_refund) {
            this.settlement_refund = settlement_refund;
        }

        public Integer getSettlement_total() {
            return settlement_total == null ? 0 : settlement_total;
        }

        public void setSettlement_total(Integer settlement_total) {
            this.settlement_total = settlement_total;
        }

        public Integer getDiscount_refund() {
            return discount_refund == null ? 0 : discount_refund;
        }

        public void setDiscount_refund(Integer discount_refund) {
            this.discount_refund = discount_refund;
        }

        public String getCurrency() {
            return currency == null ? "" : currency.trim();
        }

        public void setCurrency(String currency) {
            this.currency = currency;
        }

        public Integer getRefund_fee() {
            return refund_fee == null ? 0 : refund_fee;
        }

        public void setRefund_fee(Integer refund_fee) {
            this.refund_fee = refund_fee;
        }

    }

    /**
     * 优惠退款信息
     */
    public static class PromotionDetail {
        /**
         * 券ID
         */
        private String promotion_id;
        /**
         * 优惠范围
         */
        private RefundPromotionDetailScopeEnum scope;
        /**
         * 优惠类型
         */
        private RefundPromotionDetailTypeEnum type;
        /**
         * 优惠券面额<br/>
         * 用户享受优惠的金额（优惠券面额=微信出资金额+商家出资金额+其他出资方金额 ），单位为分
         */
        private Integer amount;
        /**
         * 优惠退款金额<br/>
         * 优惠退款金额<=退款金额，退款金额-代金券或立减优惠退款金额为用户支付的现金，说明详见代金券或立减优惠，单位为分
         */
        private Integer refund_amount;
        /**
         * 商品列表<br/>
         * 优惠商品发生退款时返回商品信息
         */
        private List<GoodsDetail> goods_detail;

        public String getPromotion_id() {
            return promotion_id == null ? "" : promotion_id.trim();
        }

        public void setPromotion_id(String promotion_id) {
            this.promotion_id = promotion_id;
        }

        public RefundPromotionDetailScopeEnum getScope() {
            return scope;
        }

        public void setScope(RefundPromotionDetailScopeEnum scope) {
            this.scope = scope;
        }

        public RefundPromotionDetailTypeEnum getType() {
            return type;
        }

        public void setType(RefundPromotionDetailTypeEnum type) {
            this.type = type;
        }

        public Integer getAmount() {
            return amount == null ? 0 : amount;
        }

        public void setAmount(Integer amount) {
            this.amount = amount;
        }

        public Integer getRefund_amount() {
            return refund_amount == null ? 0 : refund_amount;
        }

        public void setRefund_amount(Integer refund_amount) {
            this.refund_amount = refund_amount;
        }

        public List<GoodsDetail> getGoods_detail() {
            return goods_detail == null ? new ArrayList<>() : goods_detail;
        }

        public void setGoods_detail(List<GoodsDetail> goods_detail) {
            this.goods_detail = goods_detail;
        }

    }


}
